@@ -16,12 +16,17 @@ use futures::{
     pin_mut, Future, FutureExt, StreamExt,
 };
 use log::{debug, error, info, trace, warn};
-use std::{collections::HashSet, convert::TryFrom};
+use std::{
+    collections::HashSet,
+    convert::TryFrom,
+    io::{Read, Write},
+    marker::PhantomData,
+};
 
 mod backup;
 mod collection;
 
-pub use backup::{_UnitLoader as UnitLoader, _UnitSaver as UnitSaver};
+use backup::{UnitLoader, UnitSaver};
 #[cfg(feature = "initial_unit_collection")]
 use collection::{Collection, IO as CollectionIO};
 pub use collection::{NewestUnitResponse, Salt};
@@ -105,13 +110,14 @@ impl<H: Hasher, D: Data, S: Signature> TryFrom<UnitMessage<H, D, S>>
     }
 }
 
-struct Runway<'a, H, D, MK, DP, FH>
+struct Runway<'a, H, D, US, DP, FH, MK>
 where
     H: Hasher,
     D: Data,
-    MK: MultiKeychain,
+    US: Write,
     DP: DataProvider<D>,
     FH: FinalizationHandler<D>,
+    MK: MultiKeychain,
 {
     missing_coords: HashSet<UnitCoord>,
     missing_parents: HashSet<H::Hash>,
@@ -131,12 +137,14 @@ where
     ordered_batch_rx: Receiver<Vec<H::Hash>>,
     data_provider: DP,
     finalization_handler: FH,
+    unit_saver: UnitSaver<US, H, D, MK::Signature>,
     exiting: bool,
 }
 
 struct RunwayConfig<
     H: Hasher,
     D: Data,
+    US: Write,
     DP: DataProvider<D>,
     FH: FinalizationHandler<D>,
     MK: MultiKeychain,
@@ -145,6 +153,7 @@ struct RunwayConfig<
     max_round: Round,
     data_provider: DP,
     finalization_handler: FH,
+    unit_saver: UnitSaver<US, H, D, MK::Signature>,
     alerts_for_alerter: Sender<Alert<H, D, MK::Signature>>,
     notifications_from_alerter: Receiver<ForkingNotification<H, D, MK::Signature>>,
     tx_consensus: Sender<NotificationIn<H>>,
@@ -157,16 +166,17 @@ struct RunwayConfig<
     resolved_requests: Sender<Request<H>>,
 }
 
-impl<'a, H, D, MK, DP, FH> Runway<'a, H, D, MK, DP, FH>
+impl<'a, H, D, US, DP, FH, MK> Runway<'a, H, D, US, DP, FH, MK>
 where
     H: Hasher,
     D: Data,
-    MK: MultiKeychain,
+    US: Write,
     DP: DataProvider<D>,
     FH: FinalizationHandler<D>,
+    MK: MultiKeychain,
 {
     fn new(
-        config: RunwayConfig<H, D, DP, FH, MK>,
+        config: RunwayConfig<H, D, US, DP, FH, MK>,
         keychain: &'a MK,
         validator: &'a Validator<'a, MK>,
     ) -> Self {
@@ -176,6 +186,7 @@ where
             max_round,
             data_provider,
             finalization_handler,
+            unit_saver,
             alerts_for_alerter,
             notifications_from_alerter,
             tx_consensus,
@@ -204,6 +215,7 @@ where
             ordered_batch_rx,
             data_provider,
             finalization_handler,
+            unit_saver,
             responses_for_collection,
             session_id,
             exiting: false,
@@ -249,9 +261,6 @@ where
                 }
                 Response::NewestUnit(response) => {
                     trace!(target: "AlephBFT-runway", "{:?} Response newest unit received from {:?}.", self.index(), response.index());
-                    if let Some(unit) = response.as_signable().included_unit() {
-                        self.on_unit_received(unit.clone(), false)
-                    }
                     if self
                         .responses_for_collection
                         .unbounded_send(response)
@@ -461,12 +470,23 @@ where
         }
     }
 
+    fn save_unit(&mut self, uu: UncheckedSignedUnit<H, D, MK::Signature>) {
+        let h = uu.as_signable().hash();
+        trace!(target: "AlephBFT-runway", "{:?} Saving a created unit {:?}.", self.index(), h);
+        if let Err(err) = self.unit_saver.save(uu) {
+            error!(target: "AlephBFT-runway", "{:?} Failed to save unit {:?}. {}", self.index(), h, err);
+        } else {
+            trace!(target: "AlephBFT-runway", "{:?} Saved a created unit {:?}.", self.index(), h);
+        }
+    }
+
     async fn on_create(&mut self, u: PreUnit<H>) {
         debug!(target: "AlephBFT-runway", "{:?} On create notification.", self.index());
         let data = self.data_provider.get_data().await;
         let full_unit = FullUnit::new(u, data, self.session_id);
         let signed_unit = Signed::sign(full_unit, self.keychain).await;
-        self.store.add_unit(signed_unit.clone(), false);
+        self.save_unit(signed_unit.clone().into());
+        self.store.add_unit(signed_unit, false);
     }
 
     fn on_alert_notification(&mut self, notification: ForkingNotification<H, D, MK::Signature>) {
@@ -609,8 +629,15 @@ where
         self.send_consensus_notification(NotificationIn::NewUnits(units_to_move))
     }
 
-    async fn run(mut self, mut exit: oneshot::Receiver<()>) {
+    async fn run(
+        mut self,
+        units_from_backup: oneshot::Receiver<Vec<UncheckedSignedUnit<H, D, MK::Signature>>>,
+        mut exit: oneshot::Receiver<()>,
+    ) {
         let index = self.index();
+        let units_from_backup = units_from_backup.fuse();
+        pin_mut!(units_from_backup);
+
         info!(target: "AlephBFT-runway", "{:?} Runway started.", index);
         loop {
             futures::select! {
@@ -640,6 +667,18 @@ where
                     }
                 },
 
+                message = units_from_backup => match message {
+                    Ok(units) => {
+                        for u in units {
+                            self.on_unit_received(u, false);
+                        }
+                    },
+                    Err(e) => {
+                        error!(target: "AlephBFT-runway", "{:?} Units message from backup channel closed: {:?}", index, e);
+                        break;
+                    },
+                },
+
                 batch = self.ordered_batch_rx.next() => match batch {
                     Some(batch) => self.on_ordered_batch(batch).await,
                     None => {
@@ -649,7 +688,7 @@ where
                 },
 
                 _ = &mut exit => {
-                    info!(target: "AlephBFT-runway", "{:?} received exit signal", self.index());
+                    info!(target: "AlephBFT-runway", "{:?} received exit signal", index);
                     self.exiting = true;
                 }
             };
@@ -665,7 +704,7 @@ where
     }
 }
 
-pub(crate) struct RunwayIO<H: Hasher, D: Data, MK: MultiKeychain> {
+pub(crate) struct NetworkIO<H: Hasher, D: Data, MK: MultiKeychain> {
     pub(crate) alert_messages_for_network: Sender<(
         AlertMessage<H, D, MK::Signature, MK::PartialMultisignature>,
         Recipient,
@@ -683,7 +722,7 @@ fn initial_unit_collection<'a, H: Hasher, D: Data, MK: MultiKeychain>(
     validator: &'a Validator<'a, MK>,
     threshold: NodeCount,
     unit_messages_for_network: &Sender<RunwayNotificationOut<H, D, MK::Signature>>,
-    starting_round_sender: oneshot::Sender<Round>,
+    unit_collection_sender: oneshot::Sender<Round>,
     responses_from_runway: Receiver<
         UncheckedSigned<NewestUnitResponse<H, D, MK::Signature>, MK::Signature>,
     >,
@@ -697,7 +736,7 @@ fn initial_unit_collection<'a, H: Hasher, D: Data, MK: MultiKeychain>(
         return Err(());
     };
     let collection = CollectionIO::new(
-        starting_round_sender,
+        unit_collection_sender,
         responses_from_runway,
         resolved_requests,
         collection,
@@ -716,20 +755,63 @@ fn trivial_start(
     Ok(async {})
 }
 
-pub(crate) async fn run<H, D, MK, DP, FH, SH>(
+pub struct RunwayIO<
+    H: Hasher,
+    D: Data,
+    S: Signature,
+    US: Write,
+    UL: Read,
+    DP: DataProvider<D>,
+    FH: FinalizationHandler<D>,
+> {
+    pub data_provider: DP,
+    pub finalization_handler: FH,
+    pub unit_saver: UnitSaver<US, H, D, S>,
+    pub unit_loader: UnitLoader<UL, H, D, S>,
+    _phantom: PhantomData<(H, D, S)>,
+}
+
+impl<
+        H: Hasher,
+        D: Data,
+        S: Signature,
+        US: Write,
+        UL: Read,
+        DP: DataProvider<D>,
+        FH: FinalizationHandler<D>,
+    > RunwayIO<H, D, S, US, UL, DP, FH>
+{
+    pub fn new(
+        data_provider: DP,
+        finalization_handler: FH,
+        unit_saver: US,
+        unit_loader: UL,
+    ) -> Self {
+        RunwayIO {
+            data_provider,
+            finalization_handler,
+            unit_saver: UnitSaver::new(unit_saver),
+            unit_loader: UnitLoader::new(unit_loader),
+            _phantom: PhantomData,
+        }
+    }
+}
+
+pub(crate) async fn run<H, D, US, UL, MK, DP, FH, SH>(
     config: Config,
-    data_provider: DP,
-    finalization_handler: FH,
+    runway_io: RunwayIO<H, D, MK::Signature, US, UL, DP, FH>,
     keychain: MK,
     spawn_handle: SH,
-    runway_io: RunwayIO<H, D, MK>,
+    network_io: NetworkIO<H, D, MK>,
     mut exit: oneshot::Receiver<()>,
 ) where
     H: Hasher,
     D: Data,
-    MK: MultiKeychain,
+    US: Write,
+    UL: Read,
     DP: DataProvider<D>,
     FH: FinalizationHandler<D>,
+    MK: MultiKeychain,
     SH: SpawnHandle,
 {
     let (tx_consensus, consensus_stream) = mpsc::unbounded();
@@ -744,8 +826,8 @@ pub(crate) async fn run<H, D, MK, DP, FH, SH>(
     };
     let (alerter_exit, exit_stream) = oneshot::channel();
     let alerter_keychain = keychain.clone();
-    let alert_messages_for_network = runway_io.alert_messages_for_network;
-    let alert_messages_from_network = runway_io.alert_messages_from_network;
+    let alert_messages_for_network = network_io.alert_messages_for_network;
+    let alert_messages_from_network = network_io.alert_messages_from_network;
     let alerter_handle = spawn_handle.spawn_essential("runway/alerter", async move {
         alerts::run(
             alerter_keychain,
@@ -780,48 +862,69 @@ pub(crate) async fn run<H, D, MK, DP, FH, SH>(
     let mut consensus_handle = consensus_handle.fuse();
 
     let index = keychain.index();
-
     let threshold = (keychain.node_count() * 2) / 3 + NodeCount(1);
     let validator = Validator::new(config.session_id, &keychain, config.max_round, threshold);
     let (responses_for_collection, responses_from_runway) = mpsc::unbounded();
+    let (unit_collections_sender, unit_collection_result) = oneshot::channel();
+    let (loaded_units_tx, loaded_units_rx) = oneshot::channel();
+
+    let backup_loading_handle = backup::run_loading_mechanism(
+        runway_io.unit_loader,
+        index,
+        config.session_id,
+        loaded_units_tx,
+        starting_round_sender,
+        unit_collection_result,
+    )
+    .fuse();
+    pin_mut!(backup_loading_handle);
+
     #[cfg(feature = "initial_unit_collection")]
     let starting_round_handle = match initial_unit_collection(
         &keychain,
         &validator,
         threshold,
-        &runway_io.unit_messages_for_network,
-        starting_round_sender,
+        &network_io.unit_messages_for_network,
+        unit_collections_sender,
         responses_from_runway,
-        runway_io.resolved_requests.clone(),
+        network_io.resolved_requests.clone(),
     ) {
         Ok(handle) => handle.fuse(),
         Err(_) => return,
     };
     #[cfg(not(feature = "initial_unit_collection"))]
-    let starting_round_handle = match trivial_start(starting_round_sender) {
+    let starting_round_handle = match trivial_start(unit_collections_sender) {
         Ok(handle) => handle.fuse(),
         Err(_) => return,
     };
     pin_mut!(starting_round_handle);
 
+    let RunwayIO {
+        data_provider,
+        finalization_handler,
+        unit_saver,
+        ..
+    } = runway_io;
+
     let runway_config = RunwayConfig {
         data_provider,
         finalization_handler,
+        unit_saver,
         alerts_for_alerter,
         notifications_from_alerter,
         tx_consensus,
         rx_consensus,
-        unit_messages_from_network: runway_io.unit_messages_from_network,
-        unit_messages_for_network: runway_io.unit_messages_for_network,
+        unit_messages_from_network: network_io.unit_messages_from_network,
+        unit_messages_for_network: network_io.unit_messages_for_network,
         ordered_batch_rx,
         responses_for_collection,
-        resolved_requests: runway_io.resolved_requests,
+        resolved_requests: network_io.resolved_requests,
         session_id: config.session_id,
         max_round: config.max_round,
     };
     let (runway_exit, exit_stream) = oneshot::channel();
     let runway = Runway::new(runway_config, &keychain, &validator);
-    let runway_handle = runway.run(exit_stream).fuse();
+    let runway_handle = runway.run(loaded_units_rx, exit_stream).fuse();
     pin_mut!(runway_handle);
 
     loop {
@@ -841,6 +944,9 @@ pub(crate) async fn run<H, D, MK, DP, FH, SH>(
             _ = starting_round_handle => {
                 debug!(target: "AlephBFT-runway", "{:?} Starting round task terminated.", index);
             },
+            _ = backup_loading_handle => {
+                debug!(target: "AlephBFT-runway", "{:?} Backup loading task terminated.", index);
+            },
             _ = &mut exit => break,
         }
     }
